[PATCH 1/5] strace-graph: Set use strict and use warnings

Set the use strict and use warnings options.

Fixes for errors detected by strict/warnings.  $result is really a string, it
can be "-1" followed by an error text.
---
 strace-graph | 18 ++++++++++++------
 1 file changed, 12 insertions(+), 6 deletions(-)

diff --git a/strace-graph b/strace-graph
index d57e768..169c82c 100755
--- a/strace-graph
+++ b/strace-graph
@@ -33,7 +33,11 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 # THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+use strict;
+use warnings;
+
 my %unfinished;
+my $floatform;
 
 # Scales for strace slowdown.  Make configurable!
 my $scale_factor = 3.5;
@@ -41,6 +45,7 @@ my $scale_factor = 3.5;
 while (<>) {
     my ($pid, $call, $args, $result, $time);
     chop;
+    $floatform = 0;
 
     s/^(\d+)\s+//;
     $pid = $1;
@@ -119,13 +124,14 @@ sub parse_one {
     my ($in) = @_;
 
     if ($in =~ s/^\"//) {
+	my $tmp;
 	($tmp, $in) = parse_str($in);
 	if (not defined $tmp) {
 	    print STDERR "$0: $ARGV: $.: cannot parse string.\n";
 	    return (undef, $in);
 	}
 	return ($tmp, $in);
-    } elsif ($in =~ s/^0x(\x+)//) {
+    } elsif ($in =~ s/^0x([[:xdigit:]]+)//) {
 	return (hex $1, $in);
     } elsif ($in =~ s/^(\d+)//) {
 	return (int $1, $in);
@@ -214,10 +220,10 @@ sub handle_trace {
     }
 
     if ($call eq 'execve') {
-	return if $result != 0;
+	return if $result ne '0';
 
 	my ($filename, $argv) = parseargs($args);
-	($basename) = $filename =~ m/([^\/]*)$/;
+	my ($basename) = $filename =~ m/([^\/]*)$/;
 	if ($basename ne $$argv[0]) {
 	    $$argv[0] = "$basename($$argv[0])";
         }
@@ -249,7 +255,7 @@ sub straight_seq {
     my ($pid) = @_;
     my $seq = $pr{$pid}{seq};
 
-    for $elem (@$seq) {
+    for my $elem (@$seq) {
 	if ($$elem[0] eq 'EXEC') {
 	    my $argv = $$elem[2];
 	    print "$$elem[0] $$elem[1] @$argv\n";
@@ -265,7 +271,7 @@ sub first_exec {
     my ($pid) = @_;
     my $seq = $pr{$pid}{seq};
 
-    for $elem (@$seq) {
+    for my $elem (@$seq) {
 	if ($$elem[0] eq 'EXEC') {
 	    return $elem;
 	}
@@ -293,7 +299,7 @@ sub display_pid_trace {
 	}
     }
 
-    for $elem (@seq) {
+    for my $elem (@seq) {
 	$i++;
 	if ($$elem[0] eq 'EXEC') {
 	    my $argv = $$elem[2];
-- 


[PATCH 2/5] strace-graph: Handle strace -T

strace-T appends ' <time.spent>' to the line.  Tell strace-graph to ignore this
extra field.
---
 strace-graph | 5 ++++-
 1 file changed, 4 insertions(+), 1 deletion(-)

diff --git a/strace-graph b/strace-graph
index 169c82c..b09c928 100755
--- a/strace-graph
+++ b/strace-graph
@@ -43,7 +43,7 @@ my $floatform;
 my $scale_factor = 3.5;
 
 while (<>) {
-    my ($pid, $call, $args, $result, $time);
+    my ($pid, $call, $args, $result, $time, $time_spent);
     chop;
     $floatform = 0;
 
@@ -88,6 +88,9 @@ while (<>) {
     }
 
     ($call, $args, $result) = /(\S+)\((.*)\)\s+= (.*)$/;
+    if ($result =~ /^(.*) <([0-9.]*)>$/) {
+	($result, $time_spent) = ($1, $2);
+    }
     unless (defined $result) {
 	print STDERR "$0: $ARGV: $.: cannot parse line.\n";
 	next;
-- 


[PATCH 3/5] strace-graph: Handle recent strace output

--- SIGxxx does not include '(' or ')' characters, fix the regex for --SIG.

Add a regex for '+++ exited with <n> +++'. Ignore the line.

Handle syscall exit_group as well as _exit.
---
 strace-graph | 10 ++++++++--
 1 file changed, 8 insertions(+), 2 deletions(-)

diff --git a/strace-graph b/strace-graph
index b09c928..ae355aa 100755
--- a/strace-graph
+++ b/strace-graph
@@ -75,7 +75,7 @@ while (<>) {
 	delete $unfinished{$pid};
     }
 
-    if (/^--- SIG(\S+) \(.*\) ---$/) {
+    if (/^--- SIG(\S+) (.*) ---$/) {
 	# $pid received signal $1
 	# currently we don't do anything with this
 	next;
@@ -87,6 +87,12 @@ while (<>) {
 	next;
     }
 
+    if (/^\+\+\+ exited with (\d+) \+\+\+$/) {
+	# $pid exited $1
+	# currently we don't do anything with this
+	next;
+    }
+
     ($call, $args, $result) = /(\S+)\((.*)\)\s+= (.*)$/;
     if ($result =~ /^(.*) <([0-9.]*)>$/) {
 	($result, $time_spent) = ($1, $2);
@@ -244,7 +250,7 @@ sub handle_trace {
 	push @$seq, ['FORK', $result];
 	$pr{$pid}{seq} = $seq;
 	$pr{$result}{parent} = $pid;
-    } elsif ($call eq '_exit') {
+    } elsif ($call eq '_exit' || $call eq 'exit_group') {
 	$pr{$pid}{end} = $time if defined $time;
     }
 }
-- 


[PATCH 4/5] strace-graph: Cope with clone immediately followed by
 exit

A clone() followed by exit() with no intervening fork or execve resulted in the
new pid having no seq key.  That breaks when trying to use seq as an array
reference.  Ensure that clone populates the new pid with an empty seq entry.
---
 strace-graph | 3 ++-
 1 file changed, 2 insertions(+), 1 deletion(-)

diff --git a/strace-graph b/strace-graph
index ae355aa..680eb5f 100755
--- a/strace-graph
+++ b/strace-graph
@@ -212,7 +212,7 @@ my $depth = "";
 # process info, indexed by pid.
 # fields:
 #    parent         pid number
-#    seq            forks and execs for this pid, in sequence  (array)
+#    seq            clones, forks and execs for this pid, in sequence  (array)
 
 #  filename and argv (from latest exec)
 #  basename (derived from filename)
@@ -250,6 +250,7 @@ sub handle_trace {
 	push @$seq, ['FORK', $result];
 	$pr{$pid}{seq} = $seq;
 	$pr{$result}{parent} = $pid;
+	$pr{$result}{seq} = [];
     } elsif ($call eq '_exit' || $call eq 'exit_group') {
 	$pr{$pid}{end} = $time if defined $time;
     }
-- 


[PATCH 5/5] strace-graph: Print the pid in the graph

---
 strace-graph | 4 ++--
 1 file changed, 2 insertions(+), 2 deletions(-)

diff --git a/strace-graph b/strace-graph
index 680eb5f..5435e86 100755
--- a/strace-graph
+++ b/strace-graph
@@ -314,10 +314,10 @@ sub display_pid_trace {
 	if ($$elem[0] eq 'EXEC') {
 	    my $argv = $$elem[2];
 	    if (defined $elapsed) {
-		print "$lead [$elapsed] @$argv\n";
+		print "$lead [$elapsed] $pid @$argv\n";
 		undef $elapsed;
 	    } else {
-		print "$lead @$argv\n";
+		print "$lead $pid @$argv\n";
 	    }
 	} elsif ($$elem[0] eq 'FORK') {
 	    if ($i == 1) {
-- 

